<?php 
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

add_filter('woocommerce_product_data_tabs', 'wdsc_schedule_promotion_woocommerce_tab' );
function wdsc_schedule_promotion_woocommerce_tab( $tabs ) {      
	$tabs['extra-discount'] = array(
		'label'    => esc_html__('Extra Discount', 'wdsc'),
		'target'   => 'schedule_promotion_metas',
		'class'    => array('show_if_simple'),
		'priority' => 2,
	);
	return $tabs;       
}

/*
* Tab content
*/
add_action( 'woocommerce_product_data_panels', 'wdsc_schedule_promotion_metas' );
function wdsc_schedule_promotion_metas() {
	?>
	<div id="schedule_promotion_metas" class="panel woocommerce_options_panel hidden">
		<h3><?php echo esc_html__('Status', 'wdsc'); ?></h3>
		<?php
		// Add nonce field
		wp_nonce_field('wdsc_product_meta', 'wdsc_product_meta_nonce');

		woocommerce_wp_checkbox( array(
			'type'              => 'date',
			'id'                => 'wdsc_schedule_promotion_status',
			'value'             => get_post_meta( get_the_ID(), 'wdsc_schedule_promotion_status', true ),
			'label'             => esc_html__('Enable the Discount', 'wdsc'),
			'description'       => esc_html__('Activate only if the fields below have been filled in. (The coupon will self-generate only if you enable this option). It is not recommended to modify the fields below while the campaign is active.', 'wdsc')
		) );

		// START DATE TIME 
		?>
		<h3><?php echo esc_html__('Start Campaign', 'wdsc'); ?></h3>
		<?php
		woocommerce_wp_text_input( array(
			'type'              => 'date',
			'id'                => 'wdsc_schedule_promotion_start_date',
			'value'             => get_post_meta( get_the_ID(), 'wdsc_schedule_promotion_start_date', true ),
			'label'             => esc_html__('Campaign Start Date', 'wdsc'),
		) );

		woocommerce_wp_text_input( array(
			'type'              => 'time',
			'id'                => 'wdsc_schedule_promotion_start_time',
			'value'             => get_post_meta( get_the_ID(), 'wdsc_schedule_promotion_start_time', true ),
			'label'             => esc_html__('Campaign Start Time', 'wdsc'),
		) ); 
		?>
		<h3><?php echo esc_html__('End Campaign', 'wdsc'); ?></h3>
		<?php
		// END DATE TIME
		woocommerce_wp_text_input( array(
			'type'              => 'date',
			'id'                => 'wdsc_schedule_promotion_end_date',
			'value'             => get_post_meta( get_the_ID(), 'wdsc_schedule_promotion_end_date', true ),
			'label'             => esc_html__('Campaign End Date', 'wdsc'),
		) );

		woocommerce_wp_text_input( array(
			'type'              => 'time',
			'id'                => 'wdsc_schedule_promotion_end_time',
			'value'             => get_post_meta( get_the_ID(), 'wdsc_schedule_promotion_end_time', true ),
			'label'             => esc_html__('Campaign End Time', 'wdsc'),
		) ); 
		?>
		<h3><?php echo esc_html__('Discount Details', 'wdsc'); ?></h3>

		<?php
		woocommerce_wp_select(
			array(
				'id' => 'wdsc_schedule_promotion_type', 
				'label' => esc_html__('Discount Type', 'wdsc'),
				'options' => array(
					'fixed_product' => esc_html__('Fixed Product Discount', 'wdsc'),
					'percent'       => esc_html__('Percentage Discount', 'wdsc'),
				),
				'value' => get_post_meta( get_the_ID(), 'wdsc_schedule_promotion_type', true ),
			)
		);

		woocommerce_wp_text_input( array(
			'type'              => 'number',
			'id'                => 'wdsc_schedule_promotion_value',
			'value'             => get_post_meta( get_the_ID(), 'wdsc_schedule_promotion_value', true ),
			'label'             => esc_html__('Extra Discount Amount', 'wdsc'),
			'description'       => esc_html__('Fixed Discount Amount', 'wdsc'),
		));

		woocommerce_wp_text_input( array(
			'type'              => 'number',
			'id'                => 'wdsc_schedule_promotion_percent',
			'value'             => get_post_meta( get_the_ID(), 'wdsc_schedule_promotion_percent', true ),
			'label'             => esc_html__('Extra Percentage Discount', 'wdsc'),
			'description'       => esc_html__('Percentage (1-100%)', 'wdsc'),
		));

		woocommerce_wp_text_input( array(
			'type'              => 'text',
			'id'                => 'wdsc_schedule_promotion_coupon',
			'value'             => get_post_meta( get_the_ID(), 'wdsc_schedule_promotion_coupon', true ),
			'label'             => esc_html__('Generated Coupon', 'wdsc'),
			'custom_attributes' => array('readonly' => 'readonly'),
			'description'       => esc_html__('The coupon will be generated automatically.', 'wdsc'),
		)); 
		?>
	</div>

<?php 
}


/*
* Tab fields save
*/
add_action( 'woocommerce_process_product_meta', 'wdsc_schedule_promotion_save_metas', 10, 2 );
function wdsc_schedule_promotion_save_metas( $id, $post ) {
	// Verify nonce
	if ( wp_verify_nonce( isset( $_POST['wdsc_product_meta_nonce'] ) ? sanitize_text_field( $_POST['wdsc_product_meta_nonce'] ) : '', 'wdsc_product_meta' ) !== false ) {
		$wdsc_schedule_promotion_status = isset( $_POST['wdsc_schedule_promotion_status'] ) ? 'yes' : 'no';
		update_post_meta( $id, 'wdsc_schedule_promotion_status', $wdsc_schedule_promotion_status );

		if ( !empty( $_POST['wdsc_schedule_promotion_start_date'] ) ) {
			update_post_meta( $id, 'wdsc_schedule_promotion_start_date', sanitize_text_field( $_POST['wdsc_schedule_promotion_start_date'] ) );
		} else {
			delete_post_meta( $id, 'wdsc_schedule_promotion_start_date' );
		}

		if ( !empty( $_POST['wdsc_schedule_promotion_start_time'] ) ) {
			update_post_meta( $id, 'wdsc_schedule_promotion_start_time', sanitize_text_field( $_POST['wdsc_schedule_promotion_start_time'] ) );
		} else {
			delete_post_meta( $id, 'wdsc_schedule_promotion_start_time' );
		}

		if ( !empty( $_POST['wdsc_schedule_promotion_end_date'] ) ) {
			update_post_meta( $id, 'wdsc_schedule_promotion_end_date', sanitize_text_field( $_POST['wdsc_schedule_promotion_end_date'] ));
		} else {
			delete_post_meta( $id, 'wdsc_schedule_promotion_end_date' );
		}

		if ( !empty( $_POST['wdsc_schedule_promotion_end_time'] ) ) {
			update_post_meta( $id, 'wdsc_schedule_promotion_end_time', sanitize_text_field( $_POST['wdsc_schedule_promotion_end_time'] ));
		} else {
			delete_post_meta( $id, 'wdsc_schedule_promotion_end_time' );
		}

		$wdsc_schedule_promotion_value = '0';
		if ( !empty( $_POST['wdsc_schedule_promotion_value'] ) ) {
			update_post_meta( $id, 'wdsc_schedule_promotion_value', sanitize_text_field( $_POST['wdsc_schedule_promotion_value'] ));
			$wdsc_schedule_promotion_value = isset($_POST['wdsc_schedule_promotion_value']) ? sanitize_text_field($_POST['wdsc_schedule_promotion_value']) : '';
		} else {
			delete_post_meta( $id, 'wdsc_schedule_promotion_value' );
		}

		$wdsc_schedule_promotion_type = 'fixed_product';
		if ( !empty( $_POST['wdsc_schedule_promotion_type'] ) ) {
			update_post_meta( $id, 'wdsc_schedule_promotion_type', sanitize_text_field( $_POST['wdsc_schedule_promotion_type'] ));
			$wdsc_schedule_promotion_type = isset($_POST['wdsc_schedule_promotion_type']) ? sanitize_text_field($_POST['wdsc_schedule_promotion_type']) : '';
		} else {
			delete_post_meta( $id, 'wdsc_schedule_promotion_type' );
		}

		$wdsc_schedule_promotion_percent = '0';
		if ( !empty( $_POST['wdsc_schedule_promotion_percent'] ) ) {
			update_post_meta( $id, 'wdsc_schedule_promotion_percent', sanitize_text_field( $_POST['wdsc_schedule_promotion_percent'] ));
			$wdsc_schedule_promotion_percent = isset($_POST['wdsc_schedule_promotion_percent']) ? sanitize_text_field($_POST['wdsc_schedule_promotion_percent']) : '';
		} else { 
			delete_post_meta( $id, 'wdsc_schedule_promotion_percent' );
		}

		if ( !empty( $_POST['wdsc_schedule_promotion_value'] ) && ( isset( $_POST['wdsc_schedule_promotion_status'] ) && 'yes' == $_POST['wdsc_schedule_promotion_status'] ) ) {
			$coupon_code = 'EXTRA' . substr(str_shuffle(str_repeat('ABCDEFGHIJKLMNOPQRSTUVXYZ', 5)), 0, 5);
			// Generate a coupon
			wdsc_schedule_promotion_coupon_generate($id, sanitize_text_field($_POST['wdsc_schedule_promotion_end_date']), sanitize_text_field($_POST['wdsc_schedule_promotion_end_time']), $wdsc_schedule_promotion_value, $coupon_code, $discount_type);
		} elseif ( !empty( $_POST['wdsc_schedule_promotion_percent'] ) && ( isset( $_POST['wdsc_schedule_promotion_status'] ) && 'yes' == $_POST['wdsc_schedule_promotion_status'] ) ) {
			$coupon_code = 'EXTRA' . sanitize_text_field($_POST['wdsc_schedule_promotion_percent']) . substr(str_shuffle(str_repeat('ABCDEFGHIJKLMNOPQRSTUVXYZ', 5)), 0, 5);
			// Generate a coupon
			wdsc_schedule_promotion_coupon_generate($id, sanitize_text_field($_POST['wdsc_schedule_promotion_end_date']), sanitize_text_field($_POST['wdsc_schedule_promotion_end_time']), $wdsc_schedule_promotion_percent, $coupon_code, $discount_type);
		}
	} else {
		// Nonce is invalid, handle the error
		// For example, display an error message or redirect the user
		echo esc_html__('Invalid nonce. Form submission rejected.', 'wdsc');
	}	
}


function wdsc_schedule_promotion_coupon_generate($post_id, $endDate, $endTime, $amount, $coupon_code, $discount_type) {
	if ($post_id) {
		if (!$discount_type) {
			$discount_type = 'fixed_product';
		}
		$endDate = strtotime($endDate);
		$endTime = strtotime($endTime);

		// CREATE COUPON
		$coupon = array(
			'post_title' => esc_attr($coupon_code),
			'post_content' => '',
			'post_status' => 'publish',
			'post_author' => 1,
			'post_type' => 'shop_coupon'
		);
		$new_coupon_id = wp_insert_post($coupon);

		update_post_meta($new_coupon_id, 'discount_type', esc_attr($discount_type) );
		update_post_meta($new_coupon_id, 'coupon_amount', esc_attr($amount) );
		update_post_meta($new_coupon_id, 'individual_use', 'no');
		update_post_meta($new_coupon_id, 'product_ids', esc_attr($post_id) );
		update_post_meta($new_coupon_id, 'usage_limit_per_user', '1');
		update_post_meta($new_coupon_id, 'expiry_date', gmdate('d-m-Y', $endDate) . ' ' . gmdate('H:i:s', $endTime));
		update_post_meta($new_coupon_id, 'apply_before_tax', 'no');
		update_post_meta($new_coupon_id, 'free_shipping', 'no');
		update_post_meta($new_coupon_id, 'individual_use', 'yes');

		// Update Coupon custom fields (Current product)
		update_post_meta( $post_id, 'wdsc_schedule_promotion_coupon', esc_attr($coupon_code ) );
	}
}

function wdsc_schedule_promotion_coupon_update($post_id, $endDate, $endTime, $amount, $coupon_code) {
	if ($post_id) {
		// check if the post has metas saved so far
		$wdsc_schedule_promotion_end_date = get_post_meta( $post_id, 'wdsc_schedule_promotion_end_date', true);
		$wdsc_schedule_promotion_end_time = get_post_meta( $post_id, 'wdsc_schedule_promotion_end_time', true);
		$wdsc_schedule_promotion_value = get_post_meta( $post_id, 'wdsc_schedule_promotion_value', true);
		$wdsc_schedule_promotion_coupon = get_post_meta( $post_id, 'wdsc_schedule_promotion_coupon', true);
				
		if ( ( isset( $wdsc_schedule_promotion_end_date ) && !empty( $wdsc_schedule_promotion_end_date ) ) && ( isset($wdsc_schedule_promotion_value) && !empty( $wdsc_schedule_promotion_value ) ) ) {
			if ( ( $endDate != $wdsc_schedule_promotion_end_date ) || ( $amount != $wdsc_schedule_promotion_value ) ) {

				global $woocommerce;
				$coupon = new WC_Coupon($wdsc_schedule_promotion_coupon);
				$new_coupon_id = $coupon->get_id();

				$endDate = strtotime($endDate);
				$endTime = strtotime($endTime);

				// UPDATE COUPON METAS
				update_post_meta($new_coupon_id, 'coupon_amount', esc_attr($amount) );
				update_post_meta($new_coupon_id, 'expiry_date', gmdate('d-m-Y', $endDate) . ' ' . gmdate('H:i:s', $endTime));
			}
		}
	}
}


//Shortcode : Countdown
function wdsc_render_countdown ( $params ) {
	 // Define default values
	$defaults = array(
		'date'               => '',
		'type'               => '',
		'digits_font_size'   => '',
		'digits_line_height' => '',
		'texts_font_size'    => '',
		'texts_line_height'  => '',
		'digit_color'        => '',
		'text_color'         => '',
		'dots_color'         => '',
		'dots_font_size'     => '',
		'dots_line_height'   => ''
	);
	// Merge defaults with user-provided params
	$atts = shortcode_atts($defaults, $params);

	$date = sanitize_text_field($atts['date']);
	$type = empty($type) ?  'large' : sanitize_text_field($atts['type']);

	$digit_style = '';
	$digit_style .= empty($digit_color) ? '' : 'color:' . esc_attr(sanitize_text_field($atts['digit_color'])) . ';';
	$digit_style .= empty($digits_font_size) ? '' : 'font-size: ' . esc_attr(sanitize_text_field($atts['digits_font_size'])) . ' !important;';
	$digit_style .= empty($digits_line_height) ? '' : 'line-height: ' . esc_attr(sanitize_text_field($atts['digits_line_height'])) . ' !important;';

	$text_style = '';
	$text_style .= empty($text_color) ? '' : 'color:' . esc_attr(sanitize_text_field($atts['text_color'])) . ';';
	$text_style .= empty($texts_font_size) ? '' : 'font-size: ' . esc_attr(sanitize_text_field($atts['texts_font_size'])) . ' !important;';
	$text_style .= empty($text_line_height) ? '' : 'line-height: ' . esc_attr(sanitize_text_field($atts['text_line_height'])) . ' !important;';

	$dots_style = '';
	$dots_style .= empty($dots_color) ? '' : 'color:' . esc_attr(sanitize_text_field($atts['dots_color'])) . ';';
	$dots_style .= empty($dots_font_size) ? '' : 'font-size: ' . esc_attr(sanitize_text_field($atts['dots_font_size'])) . ' !important;';
	$dots_style .= empty($dots_line_height) ? '' : 'line-height: ' . esc_attr(sanitize_text_field($atts['dots_line_height'])) . ' !important;';

	$uniqueID = 'countdown_' . uniqid();

	$content = '<span class="scpotw-countdown ' . esc_html($type) . '"><span id="' . esc_html($uniqueID) . '" class="scpotw-countdown-content"></span></span>';
	$content .= '<script type="text/javascript">
		jQuery( document ).ready(function() {
			jQuery("#' . esc_attr($uniqueID) . '").countdown(' . json_encode(esc_js($date)) . ', function(event) {
				jQuery(this).html(
					event.strftime(
						"<div class=\'days\'>"
							+"<div class=\'days-digit\' style=\'' . esc_html($digit_style) . '\'>%D</div>"
							+"<div class=\'clearfix\'></div>"
							+"<div class=\'days-name\' style=\'' . esc_html($text_style) . '\'>' . esc_html__('days', 'wdsc') . '</div>"
						+"</div>"
						+"<span class=\'c_dot\' style=\'' . esc_html($dots_style) . '\'>&middot;</span>"
						+"<div class=\'hours\'>"
							+"<div class=\'hours-digit\'  style=\'' . esc_html($digit_style) . '\'>%H</div>"
							+"<div class=\'clearfix\'></div>"
							+"<div class=\'hours-name\' style=\'' . esc_html($text_style) . '\'>' . esc_html__('hours', 'wdsc') . '</div>"
						+"</div>"
						+"<span class=\'c_dot\' style=\'' . esc_html($dots_style) . '\'>&middot;</span>"
						+"<div class=\'minutes\'>"
							+"<div class=\'minutes-digit\' style=\'' . esc_html($digit_style) . '\'>%M</div>"
							+"<div class=\'clearfix\'></div>"
							+"<div class=\'minutes-name\' style=\'' . esc_html($text_style) . '\'>' . esc_html__('minutes', 'wdsc') . '</div>"
						+"</div>"
						+"<span class=\'c_dot\' style=\'' . esc_html($dots_style) . '\'>&middot;</span>"
						+"<div class=\'seconds\'>"
							+"<div class=\'seconds-digit\' style=\'' . esc_html($digit_style) . '\'>%S</div>"
							+"<div class=\'clearfix\'></div>"
							+"<div class=\'seconds-name\' style=\'' . esc_html($text_style) . '\'>' . esc_html__('seconds', 'wdsc') . '</div>"
						+"</div>"
					)
				);
			}).on("finish.countdown", function (event) {
				window.location.reload(false); 
			});
		});
		</script>';
		return $content;
}
add_shortcode('scpotw-countdown', 'wdsc_render_countdown');


function wdsc_schedule_promotion_info_product_page() {      
	$wdsc_schedule_promotion_status = get_post_meta( get_the_ID(), 'wdsc_schedule_promotion_status', true);
	$wdsc_schedule_promotion_start_date = get_post_meta( get_the_ID(), 'wdsc_schedule_promotion_start_date', true);
	$wdsc_schedule_promotion_start_time = get_post_meta( get_the_ID(), 'wdsc_schedule_promotion_start_time', true);
	$wdsc_schedule_promotion_end_date = get_post_meta( get_the_ID(), 'wdsc_schedule_promotion_end_date', true);
	$wdsc_schedule_promotion_end_time = get_post_meta( get_the_ID(), 'wdsc_schedule_promotion_end_time', true);
	$wdsc_schedule_promotion_type = get_post_meta( get_the_ID(), 'wdsc_schedule_promotion_type', true);
	$wdsc_schedule_promotion_value = get_post_meta( get_the_ID(), 'wdsc_schedule_promotion_value', true);
	$wdsc_schedule_promotion_percent = get_post_meta( get_the_ID(), 'wdsc_schedule_promotion_percent', true);
	$wdsc_schedule_promotion_coupon = get_post_meta( get_the_ID(), 'wdsc_schedule_promotion_coupon', true);

	global $woocommerce;
	$coupon = new WC_Coupon($wdsc_schedule_promotion_coupon);
	$discount = $coupon->get_amount();
	if (isset($discount) && !empty($discount)) {
		if ('fixed_product' == $wdsc_schedule_promotion_type ) {
			$discount = wc_price($discount);
		} else {
			$discount = '<strong>' . esc_html($discount) . '%</strong>';
		}
	}

	// dates
	$dateStart = gmdate('Y/m/d H:i:s', strtotime($wdsc_schedule_promotion_start_date . ' ' . $wdsc_schedule_promotion_start_time));
	$dateEnd = gmdate('Y/m/d H:i:s', strtotime($wdsc_schedule_promotion_end_date . ' ' . $wdsc_schedule_promotion_end_time));
	$now = current_time('timestamp');

	if (isset($wdsc_schedule_promotion_status) && 'yes' == $wdsc_schedule_promotion_status) {
		if ( ( $now >= strtotime($dateStart) && $now <= strtotime( $dateEnd ) ) ) {
			?>
			<div class="ct-schedule-promotion-product-block">
				<p><?php echo esc_html__('Buy this product and get Extra Discount Code ', 'wdsc') . '<strong>' . esc_html( $wdsc_schedule_promotion_coupon ) . '</strong>' . esc_html__(' in the amount of ', 'wdsc') . wp_kses_post( $discount ); ?></p>
				<p><?php echo esc_html__('Valid only in the following: ', 'wdsc'); ?></p>
				<?php echo do_shortcode('[scpotw-countdown date="' . esc_attr($dateEnd) . '" digit_color="#454545" digits_font_size="25px" digits_line_height="25px" text_color="#606060" texts_font_size="15px" text_line_height="15px" dots_font_size="40px"]'); ?>
			</div>
		<?php
		}
	}
}
add_action( 'woocommerce_before_add_to_cart_button', 'wdsc_schedule_promotion_info_product_page', 3);
